!#******************************************************************************
!# E.S.O. - VLT project
!#
!# who       when        what
!# --------  ----------  ----------------------------------------------
!# ebarrios  2005-02-04  Addapted from ces Alingment procedure
!#

!#************************************************************************
!#   NAME          EXTgrornt
!# 
!#   SYNOPSIS      EXTgrornt <image name>.fits
!# 
!#   DESCRIPTION 
!#               Frism alignment procedure
!#               Use a FF calibration light.    
!#               Alignment is determined by crosscorrelating segments of 
!#               the spatial profile.
!#
!#.VERSION
!# 100407	last modif
!#------------------------------------------------------------------------
!
! Variable definition:
! --------------------
!
clear/context -all
! 
define/para P1 ? I
indisk/fits {P1} align

define/local IMAGE/C/1/60 align
define/local refIMA/C/1/60 ""
define/local first/I/1/1 1

define/local tabRow/I/1/1 0
define/local I/I/1/1 0
define/local J/I/1/1 0
define/local K/I/1/1 0
define/local D/I/1/1 0
define/local REPEAT/I/1/1 0
define/local SCL/I/1/2 -5,-10
define/local WIN/I/1/2 0,0
define/local BIN/R/1/2 0.0,0.0
! 
define/local START/R/1/2 0.0,0.0
define/local NPIX/I/1/2 0,0
! 
define/local XSTART/R/1/1 0.0
define/local XCEN/R/1/1 0.0
define/local XS/R/1/1 0
define/local XE/R/1/1 0
define/local XEND/R/1/1 0.0
define/local xss/R/1/1 0
define/local xse/R/1/1 0
define/local yss/R/1/1 0
define/local yse/R/1/1 0
! 
define/local YSTART/R/1/1 0.0
define/local YEND/R/1/1 0.0
define/local YS/R/1/1 0
define/local YE/R/1/1 0
! 
define/local JUMP/I/1/1 0
define/local OK/I/1/1 0
define/local CUT/R/1/2 0.0,0.0
define/local MIS/R/1/3 0.0,0.0,0.0
define/local ERR/R/1/3 0.0,0.0,0.0
define/local ALI/R/1/3 0.0,0.0,0.0
define/local ALIERR/R/1/3 0.0,0.0,0.0
define/local ANG/R/1/1 0.0
define/local ANGE/R/1/1 0.0
define/local ANGERR/R/1/1 0.0
define/local ANGLE/R/1/1 0.0
define/local QUNO/R/1/1 0.0
define/local QDOS/R/1/1 0.0
define/local MEAN/R/1/1 0.0

!
! If display and graphic window don't exist they are created:
! -----------------------------------------------------------
!
create/display
create/graph
set/graph
set/graph pmode=1 xaxis=auto yaxis=auto


!
! Get Image size and binning
!
COPY/DK {IMAGE} NPIX NPIX
COPY/DK {IMAGE} START START
read/des {IMAGE}

XSTART = START(1)
XEND   = XSTART + NPIX(1) - 1
XS   =  START(1) + 50
XE   =  XEND - 50
! 
YSTART = START(2)
YEND   = YSTART + NPIX(2) - 1
YS   =  START(2) + 50
YE   =  YEND - 50
! 
write/out "XSTART: {XSTART}"
write/out "XEND:   {XEND}"
write/out "YSTART: {YSTART}"
write/out "YEND:   {YEND}"


!
! Table for one dimension center/gauss <scanline>,CENTER command
!
create/table CENTER 1 40 null
create/column CENTER :XSTART "px"
create/column CENTER :XEND   "px"
create/column CENTER :XCEN
create/column CENTER :YCEN
create/column CENTER :XERR
create/column CENTER :YERR
create/column CENTER :XSIG 
create/column CENTER :YSIG
create/column CENTER :XFWHM
create/column CENTER :YFWHM 
create/column CENTER :ICENT 
create/column CENTER :STATUS 
! 
write/table CENTER :XSTART @1 {XSTART} 
write/table CENTER :XEND   @1 {XEND}
!
! Input file name:
! ----------------
!
write/out "======================================"
write/out " Image {IMAGE}
write/out "--------------------------------------"
!
!
! Check if image is binned or windowed:
! -------------------------------------
!
copy/dk {IMAGE} STEP BIN
if BIN(1) .gt. 1.0 .or. BIN(2) .gt. 1.0 then
   write/out "#################################################"
   write/out "### Error: binned image.                      ###"
   write/out "### Bin-x = {BIN(1)}; Bin-y = {BIN(2)}.       ###" 
   write/out "### Procedure aborted.                        ###"
   write/out "### Repeat with unbinned file.                ###"
   write/out "#################################################"
   errsum = errsum + 1
   $rm *.bdf
   return
endif
!
! Check if image level in the range of 1000-60000 ADUs
! ----------------------------------------------------
!
xss = 1 + 5
xse = {NPIX(1)} - 5
yss = 1 + 5
yse = {NPIX(2)} - 5
statistics/image {IMAGE} [@{xss},@{yss}:@{xse},@{yse}] option=M
!verify
write/keyw rcorr  1.350000e+02,2.867000e+03
@@ kcompare gornt010:rcorr outputr 1,2 0.1
! 
CUT(1) = outputr(1)
CUT(2) = outputr(2)
write/out "{outputr(1)} {outputr(2)}"
! 
if outputr(2) .LT. 1000  then
	write/out " Low Signal"
	goto SIGNAL
elsif outputr(2) .GT. 60000 then
	write/out " Hight signal "
	goto SIGNAL
endif
!
! Display image:
! --------------
!
load {IMAGE} scale={SCL(1)},{SCL(2)} cuts={CUT(1)},{CUT(2)}
!
! Create table containing alignment analysis:
! -------------------------------------------
!
create/table align 4 {K} null
create/column align :x "px"
create/column align :shift "px"
create/column align :error "px"
create/column align :y "px"
write/out "============================="
write/out "=== Extracting  regions ==="
write/out "============================="
! 
K = 0
do I = {YS} {YE} 50
   K = K+1
   J = I + 49
   write/out "prof{K} = {IMAGE} {I},{J}"
   ave/row prof{K} = {IMAGE} {I},{J} 
   statistics/image prof{K} [{XS}:{XE}] option=MN
   ! 
   if outputr(2) .gt. 1000 .and. outputr(2) .lt. 60000 then
      XCEN = {XSTART} + {outputi(3)} 
      write/table CENTER :XCEN @1 {XCEN}
      plot prof{K}		    	
      center/gauss prof{K},CENTER >Null
      write/out "{outputr(5)} {outputr(14)}"
      if outputr(14) .gt. 1000 then
	 tabRow = {tabRow} + 1
	 if first .gt. 0 then 
            write/key refIMA/C/1/30 prof{K}
	    first = 0
	 endif
       	write/table align :y @{tabRow} {outputr(5)} 
	write/table align :x @{tabRow} {J}

       	xcorr/image {refIMA} prof{K} corr{K} 20     	
       	center/gauss corr{K} >Null
       	MIS(1) = outputr(5)
   	MIS(2) = outputr(5)/2048.*1000.
   	MIS(3) = outputr(5)/2048.*2048.
   	ERR(1) = outputr(7)
   	ERR(2) = outputr(7)/2048.*1000.
   	ERR(3) = outputr(7)/2048.*2048.
   	outputr(19) = MIS(3)
   	outputr(20) = ERR(3)
   	copy/kt outputr/R/19/2 align :shift :error @{tabRow}
      endif
   else
      D = D+1
      write/out "{D}.- Not enougth signal on prof{K}"
   endif
enddo
!
! Check table not empty. If it's so go to SIGNAL 
!
show/table align
if outputi(2) .lt. 2 goto SIGNAL
! 
read/table align
!
! Correct sign:
! -------------
compu/table align :shift = -1.0*:shift
!
! Plot the alignment values together with their errors:
! -----------------------------------------------------
!
compu/table align :err2 = :error*:error
statistics/table align :shift
!verify
write/keyw rcorr -0.0015049,0.897051,0.650967,0.271974 
@@ kcompare gornt020:rcorr outputr 1,4 0.00001
! 
if outputr(1) .gt. -1.0 .and. outputr(2) .lt. 1.0 set/graph yaxis=-1.2,1.2
set/graph xaxis=auto
plot/table align :x :shift
over/err align :x :shift :error ? n
set/graph stype=0 ltype=2
over/table align :x :shift
select/table align :error.gt.0
regress/poly align :shift :x 1
save/regress align fit
! 
write/out "Slope: {outputd(2)}"
ANGE = m$atan( {outputd(2)} )
! 
select/table align all
compu/regress align :fit = fit(#1)
set/graph stype=0 colour=4 ltype=1
over/table align :x :fit
! 
ALI(1) = (m$value(align,:fit,@{tabRow})-m$value(align,:fit,@1))
QUNO = m$value(align,:err2,@1)+m$value(align,:err2,@{tabRow})
ALIERR(1) = m$sqrt(QUNO)
ALI(2) = ALI(1)*9./10./1843.*1000.
ALIERR(2) = ALIERR(1)*9./10./1843.*1000.
ALI(3) = ALI(1)*9./10./1843.*2048.
ALIERR(3) = ALIERR(1)*9./10./1843.*2048.
! 
ANG = m$atan(ALI(1)/2048.)
QUNO = m$atan((ALI(1)+ALIERR(1))/2048.)-ANG
QDOS = ANG-m$atan((ALI(1)-ALIERR(1))/2048.)
ANGERR = (QUNO+QDOS)/2.
outputr(1) = alierr(1)
outputr(2) = alierr(2)
outputr(3) = alierr(3)
outputr(4) = ali(1)
outputr(5) = ali(2)
outputr(6) = ali(3)
outputr(7) = ang
outputr(8) = quno
outputr(9) = qdos
outputr(10) = angerr
!verify
write/keyw rcorr -
0.0374498,0.018288,0.0374539,0.738977,0.360868,0.739057,0.020674,0.00104771,0.00104771,0.00104771 
@@ kcompare gornt030:rcorr outputr 1,8 0.0001 9,10 0.000001
!
create/table horizontal 4 2 null
create/column horizontal :x1 "px"
create/column horizontal :x2 "px"
create/column horizontal :shift1 "px"
create/column horizontal :shift2 "px"
write/table horizontal :x1 @1 1.0
write/table horizontal :x1 @2 1024.0
write/table horizontal :x2 @1 1024.0
write/table horizontal :x2 @2 2048.0
compu/table horizontal :shift1 = +0.2
compu/table horizontal :shift2 = -0.2
set/graph ltype=3 colour=2
over/table horizontal :x1 :shift1
over/table horizontal :x1 :shift2
over/table horizontal :x2 :shift1
over/table horizontal :x2 :shift2
label/graph "Lower Limit" 2050.,-1.0
label/graph "Upper Limit" 2050.,1.0
!
statistics/table align :y
!verify
write/keyw rcorr 656.697,657.57,657.34,0.267467  
@@ kcompare gornt040:rcorr outputr 1,3 0.001 4,4 0.0001
!
@@ EXTrotalign {ANGE} {outputr(3)} {ALI(3)}
!verify
write/keyw rcorr 0.04,657.34,0.74,-40,40
@@ kcompare gornt050:rcorr outputr 1,3 0.005 4,5 0.1
! 
outputr(1) = alierr(1)
outputr(2) = alierr(2)
outputr(3) = alierr(3)
outputr(4) = ali(1)
outputr(5) = ali(2)
outputr(6) = ali(3)
outputr(7) = ang
outputr(8) = quno
outputr(9) = qdos
outputr(10) = angerr
!verify
write/keyw rcorr -
0.0374498,0.018288,0.0374539,0.738977,0.360868,0.739057,0.020674,0.00104771,0.00104771,0.00104771 
@@ kcompare gornt060:rcorr outputr 1,7 0.0001 8,10 0.000001
! 
write/out " Alignment:"
write/out " {ANG} +/- {ANGERR} degrees, i.e."
write/out " {ALI(2)} +/- {ALIERR(2)} px in 1000 px"
write/out " {ALI(3)} +/- {ALIERR(3)} px in 2048 px (whole range)"
return
! 
SIGNAL:
write/out "Check Grism image. Signal should be in the range of 1000 to 60000 ADUs"
outputr(1) = -9999.9
outputr(2) = 0
outputr(3) = 1000
outputr(4) = 999.0
outputr(5) = 999.0
errsum = errsum + 1

